name = 'bots'
from .bot import Bot
from .jittery import JitteryBot
from .still import StillBot
from .follow import LineFollower as FollowBot
from .extrovert import ExtrovertBot
from .realistic import RealisticBot
from .random import RandomBot